@@ -1,10 +1,11 @@
 use bytes::{Buf, Bytes, BytesMut};
 use serde::{Deserialize, Serialize};
 use tokio_util::codec::{Decoder, Encoder};
+use tracing::{error, trace};
 
 use super::config::Config;
 
-#[derive(Serialize, Deserialize, Debug)]
+#[derive(Serialize, Deserialize)]
 pub struct Frame {
     pub sport: u16,
     pub dport: u16,
@@ -13,7 +14,28 @@ pub struct Frame {
     pub data: Bytes,
 }
 
+impl std::fmt::Debug for Frame {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        f.debug_struct("Frame")
+            .field("sport", &self.sport)
+            .field("dport", &self.dport)
+            .field("flag", &self.flag)
+            .field("seq", &self.seq)
+            .field("data.len", &self.data.len())
+            .finish()
+    }
+}
+
 impl Frame {
+    pub fn new_no_data(sport: u16, dport: u16, flag: Flag, seq: u32) -> Self {
+        Self {
+            sport,
+            dport,
+            flag,
+            seq,
+            data: Bytes::new(),
+        }
+    }
     pub fn new_init(sport: u16, dport: u16, flag: Flag) -> Self {
         Self {
             sport,
@@ -65,7 +87,6 @@ pub enum Flag {
     Unset,
 }
 
-#[derive(Debug)]
 pub struct FrameDecoder {
     config: Config,
 }
@@ -76,10 +97,19 @@ impl FrameDecoder {
     }
 }
 
+impl std::fmt::Debug for FrameDecoder {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        f.debug_struct("FrameDecoder")
+            .field("id", &self.config.identifier)
+            .finish()
+    }
+}
+
 impl Decoder for FrameDecoder {
     type Item = Frame;
     type Error = std::io::Error;
 
+    #[tracing::instrument(skip(src))]
     fn decode(&mut self, src: &mut BytesMut) -> Result<Option<Self::Item>, Self::Error> {
         if src.len() < std::mem::size_of::<u64>() {
             return Ok(None);
@@ -89,6 +119,7 @@ impl Decoder for FrameDecoder {
         let length = u64::from_le_bytes(length_bytes);
 
         if length as usize > self.config.max_frame_size {
+            error!("Frame too large, dropping");
             return Err(std::io::Error::from(std::io::ErrorKind::InvalidInput));
         }
 
@@ -103,11 +134,12 @@ impl Decoder for FrameDecoder {
         let item: Self::Item = bincode::deserialize(&data)
             .map_err(|e| Self::Error::new(std::io::ErrorKind::InvalidData, e))?;
 
+        trace!("Decoded frame {:?}", item);
+
         Ok(Some(item))
     }
 }
 
-#[derive(Debug)]
 pub struct FrameEncoder {
     config: Config,
 }
@@ -118,15 +150,26 @@ impl FrameEncoder {
     }
 }
 
+impl std::fmt::Debug for FrameEncoder {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        f.debug_struct("FrameEncoder")
+            .field("id", &self.config.identifier)
+            .finish()
+    }
+}
+
 impl Encoder<Frame> for FrameEncoder {
     type Error = std::io::Error;
 
+    #[tracing::instrument(skip(item, dst))]
     fn encode(&mut self, item: Frame, dst: &mut BytesMut) -> Result<(), Self::Error> {
+        trace!("Encoding frame {:?}", item);
         let encoded = bincode::serialize(&item)
             .map_err(|e| Self::Error::new(std::io::ErrorKind::InvalidInput, e))?;
         let len = encoded.len();
 
         if len > self.config.max_frame_size {
+            error!("Frame too large, dropping");
             return Err(std::io::Error::from(std::io::ErrorKind::InvalidInput));
         }
 
